
/**
 * \file Shell.c
 *
 * 21.11.2012 psa: derived from VxWorks DebugTerminal DKM
 *
 */

#include "main.h"
#include "shell.h"
#include "utils.h"
#include "usart.h"
#include "i2c.h"


#define SHELL_BUFFER_SIZE   (200)
#define MAX_ARGS            (15)


typedef struct
{
	void (*ptrToCmd)(char* args[], int nbrOfArgs);
	char* cmd;
	void* NextCommand;
}t_cmd;


typedef struct
{
	t_cmd* firstCmd;
	int nbrOfCommands;
}t_cmdList;


#define TRACE if(Tci) printf
static  uint32_t  Tci = 1;    // Trace command TRACE = ON


//--------------------------------------------------------
// forward declaration
//--------------------------------------------------------
static const char version_string[]= {"V02.00"};

static void Shell_Ver();
static void Shell_Init(t_cmdList* cmdList);
static void Shell_Help(char* args[], int nbrOfArgs);
static void addCommand(t_cmdList* list, t_cmd* cmd);
static t_cmd* getCommand(t_cmdList* list, char* cmd);
static void read08Value(char* args[], int argc);
static void write08Value(char* args[], int argc);
static void read16Value(char* args[], int argc);
static void write16Value(char* args[], int argc);
static void read32Value(char* args[], int argc);
static void write32Value(char* args[], int argc);
static void readI2C(char* args[], int argc);
static void writeI2C(char* args[], int argc);
//static void setGPIO(char* args[], int argc);
//static void getGPIO(char* args[], int argc);
//static void MII_Bus_rd(char* args[], int argc);
//static void MII_Bus_wr(char* args[], int argc);
static void reset(char* args[], int argc);
static void setTCI(char* args[], int argc);

t_cmd _read08Value   = { read08Value,   "RB"};
t_cmd _write08Value  = { write08Value,  "WB"};
t_cmd _read16Value   = { read16Value,   "RW"};
t_cmd _write16Value  = { write16Value,  "WW"};
t_cmd _read32Value   = { read32Value,   "RL"};
t_cmd _write32Value  = { write32Value,  "WL"};
t_cmd _I2C_Bus_rd    = { readI2C,       "RI"};
t_cmd _I2C_Bus_wr    = { writeI2C,      "WI"};
//t_cmd _MII_Bus_rd    = { MII_Bus_rd,    "RMII"};
//t_cmd _MII_Bus_wr    = { MII_Bus_wr,    "WMII"};
//t_cmd _setGPIO       = { setGPIO,       "SIO"};
//t_cmd _getGPIO       = { getGPIO,       "GIO"};
t_cmd _setTCI        = { setTCI,        "TCI"};
t_cmd _reset         = { reset,         "RESET"};
t_cmd _ver           = { Shell_Ver,     "VER"};
t_cmd _help          = { Shell_Help,    "HELP"};
t_cmd _h             = { Shell_Help,    "H"};


static void Shell_Init(t_cmdList* cmdList)
{
  //------------------------------------------------------------------
  // initialize a new command  list
  //------------------------------------------------------------------
  cmdList->firstCmd = (t_cmd*)0;
  cmdList->nbrOfCommands = 0;
  //------------------------------------------------------------------
  // add all commands to the list here
  //------------------------------------------------------------------
  addCommand(cmdList, &_read08Value);
  addCommand(cmdList, &_write08Value);
  addCommand(cmdList, &_read16Value);
  addCommand(cmdList, &_write16Value);
  addCommand(cmdList, &_read32Value);
  addCommand(cmdList, &_write32Value);
  //addCommand(cmdList, &_setGPIO);
  //addCommand(cmdList, &_getGPIO);
  addCommand(cmdList, &_I2C_Bus_rd);
  addCommand(cmdList, &_I2C_Bus_wr);
  //addCommand(cmdList, &_MII_Bus_rd);
  //addCommand(cmdList, &_MII_Bus_wr);
  addCommand(cmdList, &_setTCI);
  addCommand(cmdList, &_reset);
  addCommand(cmdList, &_ver);
  addCommand(cmdList, &_help);
  addCommand(cmdList, &_h);
}


static void Shell_Help(char* args[], int nbrOfArgs)
{
  printf("==================================================================\n");
  printf(" RB &addr [n]                   - read n bytes from memory(addr)\n");
  printf(" RW &addr [n]                   - read n words from memory(addr)\n");
  printf(" RL &addr [n]                   - read n longs from memory(addr)\n");
  printf(" WB &addr &val1 [&val2]..       - write 1..n bytes to memory(addr)\n");
  printf(" WW &addr &val1 [&val2]..       - write 1..n words to memory(addr)\n");
  printf(" WL &addr &val1 [&val2]..       - write 1..n longs to memory(addr)\n");
  printf("-------------------------------------------------------------------\n");
  printf(" RB &addr S                     - read a string  from memory(addr)\n");
  printf(" WB &addr 'string'              - write a string to memory(addr)\n");
  printf("-------------------------------------------------------------------\n");
  printf(" RI &addr[:&ptr] [n]            - read n bytes from I2C-Bus\n");
  printf(" RI &addr[:&ptr] S              - read a string from I2C-Bus\n");
  printf(" WI &addr[:&ptr] &val..         - write a byte array to I2C-Bus\n");
  printf(" WI &addr[:&ptr] 'string'       - write a string to I2C-Bus\n");
//printf("-------------------------------------------------------------------\n");
//printf(" RMII &addr:ptr [n]             - read n words from MDIO-Bus\n");
//printf(" WMII &addr:ptr &val1 [&val2].. - write 1..n words over MDIO-Bus\n");
//printf("-------------------------------------------------------------------\n");
//printf(" SIO pin x                      - set GPIO(pin) {I,Z,H,L,T}\n");
//printf(" GIO pin                        - get GPIO(pin) {H,L}\n");
  printf("-------------------------------------------------------------------\n");
  printf(" TCI ON/OFF                     - trace command info [on/off]\n");
  printf(" VER                            - display version\n");
  printf(" HELP                           - display this help\n");
  printf(" RESET                          - Reboot the board\n");
  printf("==================================================================\n");
}


static void setTCI(char* args[], int argc)
{
  if (strcmp("ON",  args[1]) == 0) Tci = 1;
  if (strcmp("1",   args[1]) == 0) Tci = 1;
  if (strcmp("OFF", args[1]) == 0) Tci = 0;
  if (strcmp("0",   args[1]) == 0) Tci = 0;
  if (Tci)
  {
    printf("Trace for Command TRACE is ON\n");
  }
  else
  {
    printf("Trace Command TRACE is OFF\n");
  }
}


static void Shell_Ver()
{
  printf("=======================================\n");
  printf(" Welcome on the Terminal-Command-Shell \n");
  printf("---------------------------------------\n");
  printf(" ***    STM32F4-Discovery Board    *** \n");
  printf(" ***      (w) by Peter Sager       *** \n");
  printf("---------------------------------------\n");
  printf(" *** %s; %s; %s *** \n",version_string,__DATE__,__TIME__);
  printf("=======================================\n");
}


int Shell_PrintVersion(void)
{
  printf("\nCoOS Shell %s (%s, %s) is available\n",version_string,__DATE__,__TIME__);
  return(OK);
}

//------------------------------------------------------------------------------------------
// Get Version-ID, Build-Date and Built-Time of this Shell
//------------------------------------------------------------------------------------------
int Shell_Version(char *Version, char *Date, char *Time)
{
  static char sVer[16];
  static char sDate[16];
  static char sTime[16];
  strncpy(sVer, version_string, sizeof(sVer));
  strncpy(sDate, __DATE__, sizeof(sDate));
  strncpy(sTime, __TIME__, sizeof(sTime));
  sVer[sizeof(sVer)-1]  = 0;
  sDate[sizeof(sVer)-1] = 0;
  sTime[sizeof(sVer)-1] = 0;
  Version = sVer;
  Date    = sDate;
  Time    = sTime;
  return(OK);
}

//--------------------------------------------------------------------
// Shell Terminal Command Console
//--------------------------------------------------------------------
void Shell(void* pdata)
{
  static char input[SHELL_BUFFER_SIZE];
  char* args[MAX_ARGS+1];
  int   argc;
  t_cmdList cmdList;
  t_cmd*    cmd;
  CoTickDelay(10); //10 x 10ms = 100ms
  printf("\n");
  Shell_Ver();
  printf(">");
  fflush(0);
  //------------------------------------------------------------------
  // create and initialize list
  //------------------------------------------------------------------
  Shell_Init(&cmdList);
  memset(input, 0, sizeof(input));      // clear buffer
  //------------------------------------------------------------------
  // main loop exit with "Q"
  //------------------------------------------------------------------
  while(1)
  {
    CoTickDelay(1); //1 x 10ms
    //----------------------------------------------------------------
    // send prompt and to read string from console
    //----------------------------------------------------------------
    if(UART_GetLine(input))
    {
      if (input[0]!='\r')
      {
        //----------------------------------------------------------------
        // at first, lookup the input-buffer for a string in 'quotes'
        //----------------------------------------------------------------
        if (strtok(input, "'") != NULL)
        {
          args[MAX_ARGS]=strtok(NULL, "'");
        }
        else
        {
          args[MAX_ARGS]=NULL;
        }
        //----------------------------------------------------------------
        // convert input to uppercase and divide it into separate words
        // leading spaces and multiple spaces are ignored
        //----------------------------------------------------------------
        strupr(input);
        argc=0;
        args[argc] = strtok(input, " ");
        do
        {
          if ((strlen(args[argc])!=0) && (args[argc]!=0))
          {
            argc++;
          }
        }
        while (((argc<MAX_ARGS)) && ((args[argc]=strtok(NULL, " ")) != NULL));
        //----------------------------------------------------------------
        // append string parameter if available
        //----------------------------------------------------------------
        if (args[MAX_ARGS]!=NULL)
        {
          args[argc]=args[MAX_ARGS];
          argc++;
        }
        //----------------------------------------------------------------
        // find command and execute appropriate function
        //----------------------------------------------------------------
        if (argc > 0)
        {
          cmd = getCommand(&cmdList, args[0]);
          if(cmd != (t_cmd*)0)
          {
            cmd->ptrToCmd(args, argc);
          }
          else
          {
            printf("ERROR: Command '%s' not found, type 'H' for help.\n",args[0]);
          }
        }
      }
      printf(">");
      fflush(0);
    }
	}
}

//--------------------------------------------------------------------
// add a command to the command list
//--------------------------------------------------------------------
static void addCommand(t_cmdList* list, t_cmd* cmd)
{
  int i=0;
  t_cmd* ptrToCmd;
  //------------------------------------------------------------------
  // no command is added yet, so this is the first one
  //------------------------------------------------------------------
  if(list->nbrOfCommands == 0)
  {
  list->firstCmd = cmd;   
  }
  //------------------------------------------------------------------
  // append command at the end of the list
  //------------------------------------------------------------------
  else
  {
    ptrToCmd = list->firstCmd;
    for(i=1; i<list->nbrOfCommands; i++)
    {
      ptrToCmd = ptrToCmd->NextCommand;
    }
    ptrToCmd->NextCommand = cmd;  
  }
  //------------------------------------------------------------------
  // update counter
  //------------------------------------------------------------------
  list->nbrOfCommands++;
}


//--------------------------------------------------------------------
// lookup for a command in the command list
//--------------------------------------------------------------------
static t_cmd* getCommand(t_cmdList* list, char* cmd)
{
  int i=0;
  t_cmd* ptrToCmd;
  //------------------------------------------------------------------
  // initialize pointers
  //------------------------------------------------------------------
  ptrToCmd = list->firstCmd;
  //------------------------------------------------------------------
  // search the appropriate command
  //------------------------------------------------------------------
  for(i=0; i<(list->nbrOfCommands); i++)
  {
    if(strcmp(ptrToCmd->cmd, cmd) == 0)
    {
      return(ptrToCmd);
    }
    else
    {
      ptrToCmd = ptrToCmd->NextCommand;
    }
  }
  return((t_cmd*)0);
}

//=========================================================================
// Read from the I2C-Bus ((Command 'RI'))
//-------------------------------------------------------------------------
// Parameters: args[0] 'RI'       the used command 
//             args[1] adr[:ptr]  I2C-bus address with optional pointer 
//                                examples: 'E2' or 'B4:40' or 'A0:01C0' 
//             args[2] anz        dezimal number of bytes to read 
//             args[2] 'S'        read string until next stringend 
//-------------------------------------------------------------------------
// Output:     to console
//-------------------------------------------------------------------------
static void readI2C(char* args[], int argc)
{
  u08 dev;
  u08 plen=0;
  static u08 lplen=0;
  static u16 ptr;
  int i=0, cnt=1, blk;
  if (argc>1)
  {
    if (strlen(args[1])>3)          // if dev:ptr
    {
      *(args[1]+2) = 0;
      hex2int(args[1],dev);
      args[1]=args[1]+3;
      plen=(strlen(args[1])+1)>>1;
      lplen=plen;                   // save as last pointer length
      hex2int(args[1],ptr);
    }
    else 
    {
      *(args[1]+2) = 0;
      hex2int(args[1],dev);
    }
    if (argc>2)
    {
      if (*args[2]=='S')
      {
        i='S';
        cnt=128;
        if (plen==0) plen=lplen;
      }
      else cnt=atoi(args[2]);
    }
    //--------------------------------------------------------------------
    if (i=='S')
    {
      TRACE("Reading String from I2C-Device[0x%02X",dev);
    }
    else 
    {
      if (cnt==1) 
      {
        TRACE("Reading one Byte from I2C-Device[0x%02X",dev);
      }
      else
      {
        TRACE("Reading %d Bytes from I2C-Device[0x%02X",cnt,dev);
      }
    }
    //--------------------------------------------------------------------
    if (plen==1)
    {
      TRACE(":%02X]",ptr);
    }
    else if (plen==2)
    {
      TRACE(":%04X]",ptr);
    }
    else 
    {
      TRACE("]");
    }
    fflush(0);
    //--------------------------------------------------------------------
    while(cnt>0)
    {
      if (cnt>128) blk=128;
      else blk = cnt;
      cnt = cnt-blk;
      if (I2C_ReadTransfer(dev, (u08*)args[0], blk, ptr, plen)==OK)
      {
        args[1]=args[0];
        if (i=='S')
        {
          TRACE("\n");
          printf("'%s'",args[1]);
          ptr=ptr+strlen(args[1])+1;  // set pointer to stringend+1
        }
        else
        {
          for(i=0; i<blk; i++)
          {
            if ((i % 32)==0)
            {
              if (plen==1)
              {
                TRACE("\n[%02X:%02X]: ",dev,ptr+i);
              }
              else if (plen==2)
              {
                TRACE("\n[%02X:%04X]: ",dev,ptr+i);
              }
              else 
              {
                TRACE("\n[%02X]: ",dev);
              }
            }
            printf("%02X",*(volatile u08*)(args[1]++));
          }
          ptr=ptr+blk;
          i=0;
        }
      }
      else 
      {
        printf("ERROR: No response from I2C-Device");
        cnt=0;
      }
    }
    //--------------------------------------------------------------------
    printf("\n");
  }
  else printf("ERROR: Missing Parameter\n");
}

//=========================================================================
// Write to the I2C-Bus (Command 'WI')
//-------------------------------------------------------------------------
// Parameters: args[0] 'WI'      the used command 
//             args[1] adr[:ptr] I2C-bus address with optional pointer 
//                               examples: E2 or B4:40 or A0:01C0 
//             args[2] val       seqence of bytes to write, e.g. 1234ABCD 
//             args[2] 'string'  string to write, e.g. 'Hello World'  
//-------------------------------------------------------------------------
// Output:     to console and to I2C-Bus
//-------------------------------------------------------------------------
static void writeI2C(char* args[], int argc)
{
  u08 dev, plen=0;
  u16 ptr=0;
  int i=0, cnt=1;
  if (argc>2)
  {
    if (strlen(args[1])>3)          // if dev:ptr
    {
      *(args[1]+2) = 0;
      hex2int(args[1],dev);
      args[1]=args[1]+3;
      plen=(strlen(args[1])+1)>>1;
      hex2int(args[1],ptr);
    }
    else 
    {
      *(args[1]+2) = 0;
      hex2int(args[1],dev);
    }
    //----------------------------------------------------------------------
    if (args[MAX_ARGS]==NULL)
    {
      cnt=hex2bin(args[2],args[0],0);
    }
    else 
    {
      i='S';
      cnt=strlen(args[2])+1;
      args[0]=args[2];
    }
    if (cnt>128) cnt=128;
    //--------------------------------------------------------------------
    if (i=='S') 
    {
      TRACE("Writing String[%d] to I2C-Device[0x%02X",strlen(args[0])+1,dev);
    }
    else
    {
      if (cnt==1)
      {
        TRACE("Writing one Byte to I2C-Device[0x%02X",dev);
      }
      else
      {
        TRACE("Writing %d Bytes to I2C-Device[0x%02X",cnt,dev);
      }
    }
    //--------------------------------------------------------------------
    if (plen==1)
    {
      TRACE(":%02X]",ptr);
    }
    else if (plen==2)
    {
      TRACE(":%04X]",ptr);
    }
    else
    {
      TRACE("]");
    }
    fflush(0);
    //--------------------------------------------------------------------
    if (I2C_WriteTransfer(dev, (u08*)args[0], cnt, ptr, plen)==OK)
    {
      if (i=='S')
      {
        TRACE("\n");
        printf("'%s'\n",args[0]);
        ptr=ptr+strlen(args[0])+1;  // preset pointer to stringend
      }
      else
      {
        for(i=0;i<cnt;i++)
        {
          if ((i % 32)==0)
          {
            if (plen==1)
            {
              TRACE("\n[%02X:%02X]: ",dev,ptr);
            }
            else if (plen==2)
            {
              TRACE("\n[%02X:%04X]: ",dev,ptr);
            }
            else 
            {
              TRACE("\n[%02X]: ",dev);
            }
          }
          printf("%02X",*(volatile u08*)(args[0]++));
        }
        printf("\n");
      }
    }
    else 
    {
      printf("ERROR: No response from I2C-Device\n");
    }
    //--------------------------------------------------------------------
  }
  else printf("ERROR: Missing Parameter\n");
}

//=========================================================================
// Write bytes (8 bit) directly to memory
//-------------------------------------------------------------------------
// Parameters: args[0]          'WB'   the used command 
//             args[1]           adr   memory address (hex-digits)
//             args[2]..args[n]  val   1..64 bytes (hex-digits) to write
//             args[2] 'string'  string to write, e.g. 'Hello World'  
//-------------------------------------------------------------------------
// Output:     to console and to memory
//-------------------------------------------------------------------------
static void write08Value(char* args[], int argc)
{
  u32 addr;
  u08 value;
  int i=0;
  if (argc>2)
  {
    hex2int(args[1],addr);
    if (args[MAX_ARGS]==NULL)
    {
      TRACE("Writing %d Bytes to Addr[0x%08X]",argc-2,addr);
      for(i=0;i<(argc-2);i++)
      {
        if ((i % 16)==0) 
        {
          TRACE("\n[%08X]: ",addr);
        }
        hex2int(args[i+2],value);
        *(volatile u08*)(addr++)=value;
        printf("%02X ",value);
      }
      printf("\n");
    }
    else
    {
      TRACE("Writing String to Addr[0x%08X]\n",addr);
      printf("'%s'\n",args[2]);
      strcpy((char*)(addr), args[2]);
    }
  }
  else printf("ERROR: Missing Parameter\n");
}

//=========================================================================
// Write words (16 bit) directly to memory
//-------------------------------------------------------------------------
// Parameters: args[0]          'WW'   the used command 
//             args[1]           adr   memory address (hex-digits)
//             args[2]..args[n]  val   1..64 words (hex-digits) to write
//-------------------------------------------------------------------------
// Output:     to console and to memory
//-------------------------------------------------------------------------
static void write16Value(char* args[], int argc)
{
  u32 addr;
  u16 value;
  int i;
  if (argc>2)
  {
    hex2int(args[1],addr);
    TRACE("Writing %d Words to Addr[0x%08X]",argc-2,addr);
    for(i=0;i<(argc-2);i++)
    {
      if ((i % 8)==0) 
      {
        TRACE("\n[%08X]: ",addr);
      }
      hex2int(args[i+2],value);
      *(volatile u16*)(addr)=value;
      printf("%04X ",value);
      addr=addr+2;
    }
    printf("\n");
  }
  else printf("ERROR: Missing Parameter\n");
}


//=========================================================================
// Write longs (32 bit) directly to memory
//-------------------------------------------------------------------------
// Parameters: args[0]          'WL'   the used command 
//             args[1]           adr   memory address (hex-digits)
//             args[2]..args[n]  val   1..64 longs (hex-digits) to write
//-------------------------------------------------------------------------
// Output:     to console and to memory
//-------------------------------------------------------------------------
static void write32Value(char* args[], int argc)
{
  u32 addr;
  u32 value;
  int i;
  if (argc>2)
  {
    hex2int(args[1],addr);
    TRACE("Writing %d Longs to Addr[0x%08X]",argc-2,addr);
    for(i=0;i<(argc-2);i++)
    {
      if ((i % 8)==0)
      {
        TRACE("\n[%08X]: ",addr);
      }
      hex2int(args[i+2],value);
      *(volatile u32*)(addr)=value;
      printf("%08X ",value);
      addr=addr+4;
    }
    printf("\n");
  }
  else printf("ERROR: Missing Parameter\n");
}


//=========================================================================
// Read bytes (8 bit) or a string directly from memory
//-------------------------------------------------------------------------
// Parameters: args[0]     'RB'   the used command 
//             args[1]      adr   memory address (hex-digits)
//             args[2]      n     number (in dezimal) of bytes to read
//             args[2]     'S'    read string until next stringend 
//-------------------------------------------------------------------------
// Output:     to console
//-------------------------------------------------------------------------
static void read08Value(char* args[], int argc)
{
  u32 addr;
  u08 data;
  int i=0,cnt=1;
  if (argc>1)
  {
    hex2int(args[1],addr);
    if (argc>2)
    {
      if (*args[2]=='S')
      {
        i='S';
      }
      else cnt=atoi(args[2]);
    }
    if(i=='S')
    {
      TRACE("Reading String from Addr[0x%08X]\n",addr);
      printf("'%s'\n",(char *)(addr));
    }
    else
    {
      TRACE("Reading %d Bytes from Addr[0x%08X]",cnt,addr);
      for(i=0;i<cnt;i++)
      {
        if ((i % 16)==0)
        {
          TRACE("\n[%08X]: ",addr);
        }
        data = *(volatile u08*)(addr++);
        printf("%02X ",data);
      }
      printf("\n");
    }
  }
  else printf("ERROR: No Address\n");
}


//=========================================================================
// Read words (16 bit) directly from memory
//-------------------------------------------------------------------------
// Parameters: args[0]     'RW'   the used command 
//             args[1]      adr   memory address (hex-digits)
//             args[2]      n     number (in dezimal) of words to read
//-------------------------------------------------------------------------
// Output:     to console
//-------------------------------------------------------------------------
static void read16Value(char* args[], int argc)
{
  u32 addr;
  u16 data;
  int i,cnt=1;
  if (argc>1)
  {
    hex2int(args[1],addr);
    if (argc>2) cnt=atoi(args[2]);
    TRACE("Reading %d Words from Addr[0x%08X]",cnt,addr);
    for(i=0;i<cnt;i++)
    {
      if ((i % 8)==0)
      {
        TRACE("\n[%08X]: ",addr);
      }
      data = *(volatile u16*)(addr);
      printf("%04X ",data);
      addr = addr+2;
    }
    printf("\n");
  }
  else printf("ERROR: No Address\n");
}


//=========================================================================
// Read longs (32 bit) directly from memory
//-------------------------------------------------------------------------
// Parameters: args[0]     'RL'   the used command 
//             args[1]      adr   memory address (hex-digits)
//             args[2]      n     number (in dezimal) of longs to read
//-------------------------------------------------------------------------
// Output:     to console
//-------------------------------------------------------------------------
static void read32Value(char* args[], int argc)
{
  u32 addr;
  u32 data;
  int i,cnt=1;
  if (argc>1)
  {
    hex2int(args[1],addr);
    if (argc>2) cnt=atoi(args[2]);
    TRACE("Reading %d Longs from Addr[0x%08X]",cnt,addr);
    for(i=0;i<cnt;i++)
    {
      if ((i % 8)==0)
      {
        TRACE("\n[%08X]: ",addr);
      }
      data = *(volatile u32*)(addr);
      printf("%08X ",data);
      addr = addr+4;
    }
    printf("\n");
  }
  else printf("ERROR: No Address\n");
}

static void reset(char* args[], int argc)
{
  printf("Reboot...\n");
  CoTickDelay(10); //10 x 10ms = 100ms
  NVIC_SystemReset();
  while(1);   //wait for reset
}


#if MHCU

//=========================================================================
// Read a message from HDLC-Channel(0..7)
//-------------------------------------------------------------------------
// Parameters: args[0]     'RH'   the used command
//             args[1]      chn   HDLC Channel (0..7)
//             args[2]      n     number (in dezimal) of messages to read
//             args[3]     'S'    read string until next stringend
//-------------------------------------------------------------------------
// Output:     to console
//-------------------------------------------------------------------------
static void HDLC_Bus_rd(char* args[], int argc)
{
  u08 data;
  char rd[16];
  int i=0, cnt=1, len, chn, readfd;
  struct timeval tv;
  fd_set fdSet;
  if (argc>1)
  {
    hex2int(args[1],chn);
    if (chn < 8)
    {
      if (argc>2)
      {
        if (*args[2]=='S')
        {
          i='S';
        }
        else cnt=atoi(args[2]);
      }
      if (argc>3)
      {
        if (*args[3]=='S')
        {
          i='S';
        }
        else cnt=atoi(args[3]);
      }
      sprintf(rd, "/hdlc/%d", chn);
      readfd = open(rd, O_RDWR, 0644);
      if (readfd < 0)
      {
        printf("Error: Failed to open HDLC-Device %s\n",rd);
        return;
      }
      tv.tv_sec = 0;
      tv.tv_usec = 25000;  //timeout 25 ms
      while (cnt)
      {
        if (i=='S')
        {
          TRACE("Reading String-Message from HDLC-Device %s", rd);
        }
        else
        {
          TRACE("Reading Message from HDLC-Device %s", rd);
        }
        FD_ZERO(&fdSet);
        FD_SET(readfd, &fdSet);
        if(select(FD_SETSIZE, &fdSet, NULL, NULL, &tv) == 1)
        {
          if(FD_ISSET(readfd, &fdSet))
          {
            len = read(readfd, args[0], BUFFER_SIZE);
          }
          else
          {
            TRACE("\n");
            printf("Error: Failed reading from HDLC-Device %s\n",rd);
            close(readfd);
            return;
          }
        }
        else
        {
          TRACE("\n");
          printf("Error: Timeout on reading from HDLC-Device %s\n",rd);
          close(readfd);
          return;
        }
        if (i=='S')
        {
          TRACE(" (%d Byte)\n",len);
          *(args[0]+len) = 0;
          printf("'%s'\n",args[0]);
        }
        else
        {
          TRACE(" (%d Byte)",len);
          for(i=0;i<len;i++)
          {
            if ((i % 32)==0)
            {
              TRACE("\n");
            }
            printf("%02X",*(volatile u08*)(args[0]++));
          }
          printf("\n");
        }
        cnt--;
      }
      close(readfd);
    }
    else printf("ERROR: Invalid Channel Number (chn = 0..7)\n");
  }
  else printf("ERROR: Missing Parameter\n");
}


//=========================================================================
// Write a message to HDLC-Cannel(0..7)
//-------------------------------------------------------------------------
// Parameters: args[0]          'WH'   the used command
//             args[1]           chn   HDLC Channel (0..7)
//             args[2]..args[n]  val   1..64 bytes (hex-digits) to write
//             args[2] 'string'  string to write, e.g. 'Hello World'
//-------------------------------------------------------------------------
// Output:     to console and to memory
//-------------------------------------------------------------------------
static void HDLC_Bus_wr(char* args[], int argc)
{
  u08 value;
  char wd[16];
  int i=0, cnt=1, chn, writefd;
  if (argc>2)
  {
    hex2int(args[1],chn);
    if (chn < 8)
    {
      //=====================================================================
      sprintf(wd, "/hdlc/%d", chn);
      writefd = open(wd, O_RDWR, 0644);
      if (writefd < 0)
      {
        printf("Error: Failed to open HDLC-Device %s\n",wd);
        return;
      }
      //=====================================================================
      if (args[MAX_ARGS]==NULL)
      {
        cnt=hex2bin(args[2],args[0],0);
      }
      else
      {
        i='S';
        cnt=strlen(args[2])+1;
        args[0]=args[2];
      }
      if (cnt>256) cnt=256;  //limitation to 256 bytes
      //=====================================================================
      if (i=='S')
      {
        TRACE("Writing String[%d] to HDLC-Device %s",strlen(args[0])+1,wd);
      }
      else
      {
        if (cnt==1)
        {
          TRACE("Writing one Byte to HDLC-Device %s",wd);
        }
        else
        {
          TRACE("Writing %d Bytes to HDLC-Device %s",cnt,wd);
        }
      }
      //--------------------------------------------------------------------
      if (write(writefd, args[0], cnt) < 0)
      {
        TRACE("\n");
        printf("Error: Failed writing to HDLC-Device %s\n",wd);
      }
      else
      {
        if (i=='S')
        {
          TRACE("\n");
          printf("'%s'\n",args[0]);
        }
        else
        {
          for(i=0;i<cnt;i++)
          {
            if ((i % 32)==0)
            {
              TRACE("\n");
            }
            printf("%02X",*(volatile u08*)(args[0]++));
          }
          printf("\n");
        }
      }
      //--------------------------------------------------------------------
      close(writefd);
    }
    else printf("ERROR: Invalid Channel Number (chn = 0..7)\n");
  }
  else printf("ERROR: Missing Parameter\n");
}



//=========================================================================
// Set GPIO(pin) to Z,P,H,L,T
//-------------------------------------------------------------------------
// Parameters: args[0]      'SIO'  the used command
//             args[1]       pin   GPIO pin number (dezimal-digits)
//             args[2]       mode  'I' => Input  High-Z
//                                 'Z' => Output Open Drain
//                                 'H' => Output High
//                                 'L' => Output Low
//                                 'T' => Output Toggle Level
//-------------------------------------------------------------------------
// Output:     to console and GPIO-Signal
//-------------------------------------------------------------------------
static void setGPIO(char* args[], int argc)
{
  int pin;
  if (argc>2)
  {
    pin=atoi(args[1]);
    if ((pin>=0) && (pin<=31))
    {
      if (*args[2]=='T')
      {
        if (mpc83xxGpioGet(pin))
        {
           *args[2]='L';
        }
        else
        {
          *args[2]='H';
        }
      }
      if(*args[2]=='I')
      {
        mpc83xxGpioConfig(pin, INPUT, DONTCARE, LOW);
        TRACE("Setting GPIO_%d as Input\n",pin);
      }
      else if (*args[2]=='Z')
      {
        mpc83xxGpioConfig(pin, OUTPUT, OPENDRAIN, HIGH);
        TRACE("Setting GPIO_%d as Output(Opendrain)\n",pin);
      }
      else if ((*args[2]=='H') || (*args[2]=='1'))
      {
        mpc83xxGpioConfig(pin, OUTPUT, PUSHPULL, HIGH);
        TRACE("Setting GPIO_%d as Output(High)\n",pin);
      }
      else if ((*args[2]=='L') || (*args[2]=='0'))
      {
        mpc83xxGpioConfig(pin, OUTPUT, PUSHPULL, LOW);
        TRACE("Setting GPIO_%d as Output(Low)\n",pin);
      }
      else printf("ERROR: Invalid IO-Status\n");
    }
    else printf("ERROR: Invalid IO-Pin Number\n");
  }
  else printf("ERROR: Missing Parameter\n");
}

//=========================================================================
// Get GPIO(pin) => H,L
//-------------------------------------------------------------------------
// Parameters: args[0]      'GIO'  the used command
//             args[1]       pin   GPIO pin number (dezimal-digits)
//-------------------------------------------------------------------------
// Output:     to console GPIO_(pin) = High/Low
//-------------------------------------------------------------------------
static void getGPIO(char* args[], int argc)
{
  int pin;
  if (argc>1)
  {
    pin=atoi(args[1]);
    if ((pin>=0) && (pin<=31))
    {
      if (mpc83xxGpioGet(pin))
      {
        printf("GPIO_%d = High\n",pin);
      }
      else
      {
        printf("GPIO_%d = Low\n",pin);
      }
    }
    else printf("ERROR: Invalid IO-Pin Number\n");
  }
  else printf("ERROR: Missing Parameter\n");
}

//=========================================================================
// Read from the MII-Bus (MDIO-Read)
//-------------------------------------------------------------------------
// Parameters: args[0] 'RMII'     the used command
//             args[1] adr:ptr    Device address with register pointer
//                                example: '0F:A0'
//-------------------------------------------------------------------------
// Output:     to console
//-------------------------------------------------------------------------
static void MII_Bus_rd(char* args[], int argc)
{
  u16 dev, ptr, val;
  int i=0, cnt=1;
  if (argc>2) cnt=atoi(args[2]);
  if (argc>1)
  {
    if (strlen(args[1])>4)          // if dev:ptr
    {
      *(args[1]+2) = 0;
      hex2int(args[1],dev);
      hex2int(args[1]+3,ptr);
      TRACE("Reading %d Words from MII-Device[%02X:%02X]",cnt,dev,ptr);
      //--------------------------------------------------------------------
      for(i=0;i<cnt;i++)
      {
        if ((i % 8)==0)
        {
          TRACE("\n[%02X:%02X]: ",dev,(ptr & 0x1F));
        }
        if (mii_read(dev, ptr, &val)==OK)
        {
          printf("%04X ",val);
        }
        else
        {
          printf("#ERR ");
        }
        ptr=ptr+1;
      }
      //--------------------------------------------------------------------
      printf("\n");
    }
    else
    {
      printf("ERROR: Invalid argument\n");
    }
  }
  else printf("ERROR: Missing Parameter\n");
}

//=========================================================================
// Write the MII-Bus (MDIO-Write)
//-------------------------------------------------------------------------
// Parameters: args[0] 'WMII'    the used command
//             args[1] adr:ptr   Device address with register pointer
//                               example: '0F:A0'
//             args[2] val       Register word  to write,  12F0
//-------------------------------------------------------------------------
// Output:     to console and to MDIO-Bus
//-------------------------------------------------------------------------
static void MII_Bus_wr(char* args[], int argc)
{
  u16 dev, ptr, val;
  int i=0;
  if (argc>2)
  {
    if (strlen(args[1])>4)          // if dev:ptr
    {
      *(args[1]+2) = 0;
      hex2int(args[1],dev);
      hex2int(args[1]+3,ptr);
      TRACE("Writing %d Words to MII-Device[%02X:%02X]",argc-2,dev,ptr);
      //--------------------------------------------------------------------
      for(i=0;i<(argc-2);i++)
      {
        if ((i % 8)==0)
        {
          TRACE("\n[%02X:%02X]: ",dev,(ptr & 0x1F));
        }
        hex2int(args[i+2],val);
        if (mii_write(dev, ptr, val)==OK)
        {
          printf("%04X ",val);
        }
        else
        {
          printf("#ERR ");
        }
        ptr=ptr+1;
      }
      //--------------------------------------------------------------------
      printf("\n");
    }
    else
    {
      printf("ERROR: Invalid argument\n");
    }
  }
  else printf("ERROR: Missing Parameter\n");
}


#endif //MHCU

